﻿// описываем какие именно части сторонних библиотек будем использовать (Глва 16)

using System;

// нужно для использования типа Security
using Ecng.Trading.BusinessEntities;

// нужно для использования типа QuikTrader
using Ecng.Trading.Quik;

namespace Orders
{
	class Program
	{
		// Main - входная точка в программу
		static void Main(string[] args)
		{
			// 1. создаем переменную trader, имеющая тип QuikTrader
			// 2. создаем сложный объект типа QuikTrader (new - ключевое слово создания сложных объектов)
			// 3. присваиваем через знак "=" значение созданное на этапе 2 в переменную trader.
			// Глава 3, стр 147-155, 166-171
			QuikTrader trader = new QuikTrader();

			// создаем переменную lkoh типа Security и инициализируем ее пустым значением null.
			// переменная нужна в дальнейшем для регистрации заявки
			Security lkoh = null;

			// подписываемся на событие успешного подключения
			// Глава 15, 494-504, 488-494
			trader.Connected += () =>
			{
				// выводим в консоль сообщение
				// стр 155-166, 260-268
				Console.WriteLine("Успешное подключение");

				// тело обработчика события Connected, где мы вызываем запуск экспорта
				// стр 155-166
				trader.StartExport();
			};

			// подписываемся на событие появление новых инструментов
			// в отличие от Connected, событие NewSecurities передает коллекцию новых инструментов ввиде параметра securities
			// Глава 15, 494-504, 488-494
			trader.NewSecurities += (securities) =>
			{
				// выводим в консоль сообщение
				// стр 155-166, 260-268
				Console.WriteLine("Новые инструменты");

				// перебираем в цикле foreach все инструменты
				// стр 129-142
				foreach (Security security in securities)
				{
					// стр 155-166, 260-268, 812-813
					Console.WriteLine("{0} {1}", security.Name, security.Code);

					// если наш инструмент является Лукойлом
					// стр 121-129
					if (security.Code == "LKOH")
					{
						// присваиваем переменной lkoh
						lkoh = security;
					}
				}
			};

			// создаем переменную portfolio типа Portfolio и инициализируем ее пустым значением null.
			// переменная нужна в дальнейшем для регистрации заявки
			Portfolio portfolio = null;

			// подписываемся на событие появление новых портфелей (событие аналогично NewSecurities)
			// Глава 15, 494-504, 488-494
			trader.NewPortfolios += (pfs) =>
			{
				// выводим в консоль сообщение
				// стр 155-166, 260-268
				Console.WriteLine("Новые портфели");

				// если наш портфель уже был ранее найден, то нет смысла дальше перебирать новые портфели
				// стр 121-129
				if (portfolio != null)
					return; // стр. 143

				// перебираем в цикле foreach все портфели
				// стр 129-142
				foreach (var pf in pfs)
				{
					// стр 155-166, 260-268
					Console.WriteLine(pf.Name);

					// если наш портфель является ММВБ-шным
					// стр 121-129
					if (pf.Name == "NL0011100043")
					{
						// присваиваем переменной portfolio
						portfolio = pf;
					}
				}
			};

			// подписываемся на событие появление новых заявок (событие аналогично NewSecurities)
			// через данное событие можно узнать, зарегистрировалась ли наша заявка (ниже), и о новых заявках введенных в ручную
			// Глава 15, 494-504, 488-494
			trader.NewOrders += orders =>
			{
				// перебираем в цикле foreach все новые заявки
				// стр 129-142
				foreach (var order1 in orders)
				{
					// преобразовываем данные о новой заявке (номер, цена, объем) в строку с форматированием
					// стр 816-832
					string s = string.Format("{0} {1} {2}", order1.Id, order1.Price, order1.Volume);

					// стр 155-166, 260-268
					Console.WriteLine(s);
				}
			};

			// подписываемся на событие изменения заявок (событие аналогично NewSecurities)
			// через данное событие можно узнать о состоянии заявки (исполнилась, отменилась, частично исполнилась)
			// Глава 15, 494-504, 488-494
			trader.OrdersChanged += orders =>
			{
				// перебираем в цикле foreach все измененные заявки
				// стр 129-142
				foreach (var order1 in orders)
				{
					// преобразовываем данные об измененной заявке (номер, состояние) в строку с форматированием
					// стр 816-832
					string s = string.Format("{0} {1}", order1.Id, order1.State);

					// стр 155-166, 260-268
					Console.WriteLine(s);
				}
			};

			// подписываемся на событие появление новых собственных сделок (событие аналогично NewSecurities)
			// если выставленная ниже заявка будет исполнена, то придет инфомрация о ее сделках
			// Глава 15, 494-504, 488-494
			trader.NewMyTrades += trades =>
			{
				// перебираем в цикле foreach все новые собственные сделки
				// стр 129-142
				foreach (var trade in trades)
				{
					// преобразовываем данные о новой собственной сделке (номер сделки, цена, номер заявки) в строку с форматированием
					// стр 816-832
					string s = string.Format("{0} {1} {2}", trade.Trade.Id, trade.Trade.Price, trade.Order.Id);

					// стр 155-166, 260-268
					Console.WriteLine(s);
				}
			};

			// стр 155-166
			trader.Connect();

			// стр 155-166, 260-268
			Console.ReadLine();

			// создаем переменную order типа Order и одновременно инициализируем ее (аналогично переменной trader)
			// Глава 3, стр 147-155, 166-171
			Order order = new Order();

			// заполняем необходимые данные заявки через свойства типа Order
			// стр. 313-319
			order.Security = lkoh;
			order.Portfolio = portfolio;
			order.Volume = 1;
			order.Price = 1956.1;
			order.Direction = OrderDirections.Buy;

			// регистрируем заявки
			// стр 155-166
			trader.RegisterOrder(order);

			// если в Квике снять выставленную заявку, то по ней придет второй раз изменение через событие OrdersChanged
			// первый раз изменение будет получено в связи успешной регистрацией на бирже и присвоению номера заявке

			// стр 155-166, 260-268
			Console.ReadLine();

			// удаляем явно из памяти переменную trader (остановка экспорта + отсоединение)
			trader.Dispose();
		}
	}
}
